#!/usr/bin/python
# -*- coding: UTF-8 -*- 

#############################################################################
#  Copyright & License:
#  ====================
#   
#  Copyright 2009-2015 gAGE/UPC & ESA 
#   
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#   
#  http://www.apache.org/licenses/LICENSE-2.0
#   
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#############################################################################

#############################################################################
# Copyright: gAGE/UPC & ESA
# Project: EDUNAV GNSS Lab Tool
# Authors: Adria Rovira-Garcia (group of Astronomy and GEomatics - gAGE/UPC)
#          Pere Ramos-Bosch (group of Astronomy and GEomatics - gAGE/UPC)
# Curator: Adria Rovira-Garcia ( gAGE/UPC )
#          glab.gage @ upc.edu           
# File: graph.py
# Code Management Tool File Version: 2.0  Revision: 00
# Date: 2010/12/31
#############################################################################

#############################################################################
# MODULE DESCRIPTION
#
# Name: graph
# Language: python
#
# Purpose:
#  The purpose of this module is to generate a plot for a set of specified
#  input data.
#
# Function:
#  This program creates a window which will show a plot for the specified 
#  columns in a file. It allows to include conditions to select which files
#  to plot.
# 
# Dependencies:
#  None
#
# Files modified:
#  .plot.lock
#
# Files read:
#  None
#
# Resources usage:
#  See Design Document
#############################################################################

#############################################################################
#       RELEASE_HISTORY
# -------------
#          gLAB v1.3.0
# Release: 2009/12/10
# Change Log: First public version.
# -------------
#          gLAB v1.4.0
# Release: 2010/06/21
# Change Log: Save figure function added.
# -------------
#          gLAB v1.4.1
# Release: 2010/07/09
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.2
# Release: 2010/07/31
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.3
# Release: 2010/08/31
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.4
# Release: 2010/09/22
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.5
# Release: 2010/10/31
# Change Log: Minor Changes in help 
# -------------
#          gLAB v2.0.0
# Release: 2012/12/31
# Change Log: gLAB version released together with the Educational Book:
#             "GNSS Data Processing" (Vol. 1 and Vol. 2). ESA TM-23.
#             Authors: J. Sanz Subirana, J.M. Juan Zornoza and M. Hernandez-Pajares
#             ISBN: 978-92-9221-885-0 (two volumes)
#             ISSN: 1013-7076
#             December 2012.
# -------------
#       END_RELEASE_HISTORY
#############################################################################

import pylab,sys,os,matplotlib,math,optparse
from optparse import OptionParser

global legend
legend=False
class Plot:
    def __init__(self):
        self.FileName=""
        self.ColX="var[0]"
        self.ColY="var[1]"
        self.Cond=""
        self.Label=""
        self.Xvar = []
        self.Yvar = []
        self.Style="."
        self.MarkerSize=5
        self.LineWidth=0
        self.PlotColor=""
        self.PlotColorAuto = True

class Graphic:
    def __init__(self):
        self.PlotList=[]
        self.PlotCounter = 0
        self.Title = ""
        self.Xlabel = ""
        self.Ylabel = ""
        self.FractionalTitle = ""

        self.Xmin = 0.0
        self.Xmax = 0.0
        self.Ymin = 0.0
        self.Ymax = 0.0
        
        self.XminAuto = True
        self.XmaxAuto = True
        self.YminAuto = True
        self.YmaxAuto = True
        
        self.SaveFigure = False 
        self.SaveFigurePath = ""
        
##########################################################################
# Name        : Parse
# Description : This function reads from the command line all the 
#               options, and assigns them into the variables.
##########################################################################  
def Parse(value):
    auxValue=""
    mode=0
    try:
        number = str(int(value)-1)
        auxValue = "var["+number+"]"
    except:
        for element in value:
            if (mode==1):
                try:
                    number = str(int(element))
                    #auxValue=auxValue+number
                    n=n+number
                except:
                    mode=0
                    auxValue=auxValue+str(int(n)-1)+"]"
                    #auxValue=auxValue+"]"
            if (mode==0):
                if (element=="$"):
                    auxValue=auxValue+"var["
                    mode=1
                    n=""
                else:
                    auxValue=auxValue+element
        if (mode==1):
            auxValue=auxValue+str(int(n)-1)+"]"
    return auxValue
    
##########################################################################
# Name        : ParseCallback
# Description : When a new option for the graphic is introduced, this 
#               callback is called for storing the new parameter
##########################################################################   
def ParseCallback(option, opt_str, value, parser, Graph):

    if (option.dest=="Title"):
        #Graph.Title = value
        Graph.Title = unicode(value, "utf-8", errors="ignore")

    if (option.dest=="FractionalTitle"):
        #Graph.FractionalTitle = value
        Graph.FractionalTitle = unicode(value, "utf-8", errors="ignore")

    if (option.dest=="Xlabel"):
        #Graph.Xlabel = value
        Graph.Xlabel = unicode(value, "utf-8", errors="ignore")
    
    if (option.dest=="Ylabel"):
        #Graph.Ylabel = value
        Graph.Ylabel = unicode(value, "utf-8", errors="ignore")
    
    if (option.dest=="filename"):
        Graph.PlotCounter += 1
        Graph.PlotList.append(Plot())
        Graph.PlotList[Graph.PlotCounter-1].FileName = value
    
    if (option.dest=="ColX"):
        value = Parse(value)
        Graph.PlotList[Graph.PlotCounter-1].ColX = value
    
    if (option.dest=="ColY"):
        value = Parse(value)
        Graph.PlotList[Graph.PlotCounter-1].ColY = value
 
    if (option.dest=="Cond"):
        value = Parse(value)
        Graph.PlotList[Graph.PlotCounter-1].Cond = value

    if (option.dest=="PlotLabel"):
        global legend
        legend=True 
        #Graph.PlotList[Graph.PlotCounter-1].Label = value        
        Graph.PlotList[Graph.PlotCounter-1].Label = unicode(value, "utf-8", errors="ignore")        

    if (option.dest=="Style"):
        Graph.PlotList[Graph.PlotCounter-1].Style = value

	if (value.count('-')!=0 ): 
        	Graph.PlotList[Graph.PlotCounter-1].LineWidth = 1.0

    if (option.dest=="MarkerSize"):
        Graph.PlotList[Graph.PlotCounter-1].MarkerSize = float(value)

    if (option.dest=="LineWidth"):
        Graph.PlotList[Graph.PlotCounter-1].LineWidth = float(value)

    if (option.dest=="Xmin"):
        if (str(value) == '*' or value == "auto" ): Graph.XminAuto = True
        else:
            Graph.XminAuto = False
            Graph.Xmin = float(value)
    
    if (option.dest=="Xmax"):
        if (str(value) == '*' or value == "auto" ): Graph.XmaxAuto = True
        else:
            Graph.XmaxAuto = False
            Graph.Xmax = float(value)
        
    if (option.dest=="Ymin"):
        if (str(value) == '*' or value == "auto" ): Graph.YminAuto = True
        else:        
            Graph.YminAuto = False
            Graph.Ymin = float(value)
    
    if (option.dest=="Ymax"):
        if (str(value) == '*' or value == "auto" ): Graph.YmaxAuto = True
        else:
            Graph.YmaxAuto = False
            Graph.Ymax = float(value)

    if (option.dest=="PlotColor"):
        Graph.PlotList[Graph.PlotCounter-1].PlotColorAuto = False
        Graph.PlotList[Graph.PlotCounter-1].PlotColor = value
      
    if (option.dest=="SaveFigure"):
        Graph.SaveFigure = True 
        Graph.SaveFigurePath = str(value)   
    
        
##########################################################################
# Name        : MakeGraph
# Description : This function makes the graphic, from the parameters
#               already parsed.
##########################################################################    
def MakeGraph():
    i=0
    Xvar = []
    Yvar = []

    while i<Graph.PlotCounter:
        file=open(Graph.PlotList[i].FileName,'r') 
        Xvarlocal=[]
        Yvarlocal=[]
        for line in file:
            var=line.split()
            j=0
            for ele in var:
                try:
                    auxEle = float(ele)
                    var[j] = auxEle
                    j += 1
                except:
                    j += 1
            if (Graph.PlotList[i].Cond!=""):
                try:
                    ret = eval(Graph.PlotList[i].Cond)
                except:
                    ret = False
            else:
                ret = True
            if (ret==True):
                error = 0
                try:
                    ret1 = eval(Graph.PlotList[i].ColX)
                except:
                    error = 1
                
                try:
                    ret2 = eval(Graph.PlotList[i].ColY)
                except:
                    error = 1
                
                if (error==0):
                    Graph.PlotList[i].Xvar.append(ret1)
                    Graph.PlotList[i].Yvar.append(ret2)
        file.close()
        
        if (Graph.PlotList[i].PlotColorAuto == False): 
            pylab.plot(Graph.PlotList[i].Xvar,Graph.PlotList[i].Yvar,Graph.PlotList[i].Style,label=Graph.PlotList[i].Label,color=Graph.PlotList[i].PlotColor,markersize=Graph.PlotList[i].MarkerSize,linewidth=Graph.PlotList[i].LineWidth)
        else:
            pylab.plot(Graph.PlotList[i].Xvar,Graph.PlotList[i].Yvar,Graph.PlotList[i].Style,label=Graph.PlotList[i].Label,markersize=Graph.PlotList[i].MarkerSize,linewidth=Graph.PlotList[i].LineWidth)
        
        i+=1
    
    pylab.xlabel(Graph.Xlabel)
    pylab.ylabel(Graph.Ylabel)
    pylab.title(Graph.Title)
    if (Graph.FractionalTitle != ""):
	    pylab.annotate(Graph.FractionalTitle, xy=(.025, .975), xycoords='figure fraction',                horizontalalignment='left', verticalalignment='top')




    if (Graph.XmaxAuto == False): pylab.xlim( xmax=Graph.Xmax )
    if (Graph.XminAuto == False): pylab.xlim( xmin=Graph.Xmin )
    if (Graph.YmaxAuto == False): pylab.ylim( ymax=Graph.Ymax )
    if (Graph.YminAuto == False): pylab.ylim( ymin=Graph.Ymin )

    pylab.grid(True)
    

#######        pylab.legend(numpoints=1, borderpad=0.25,handlelength=20,columnspacing=1,ncol=2)
# ok    pylab.legend(numpoints=1, handlelength=0.2) 
#        pylab.legend(numpoints=1,  borderpad=0.2)
#DEF:
#       pylab.legend(numpoints=1, borderpad=0.2,bbox_to_anchor=(0.85, 1), lo    c=2, ncol=1)
#       pylab.legend(numpoints=1, borderpad=0.2,bbox_to_anchor=(-0.08, 0.975    ), loc=2, ncol=6, handlelength=-0.0, columnspacing=0.50)

    if (legend==True):
        pylab.legend()
        #pylab.legend(numpoints=1,bbox_to_anchor=(1.12, 1)) #Use this to align the label to the upper right corner (outside the plot)
        #pylab.legend(numpoints=1,bbox_to_anchor=(1, 1)) #Use this to align the label to the upper right corner (inside the plot)
        #pylab.legend(numpoints=1,bbox_to_anchor=(0.32, 1)) #Use this to align the label to the upper left corner (after the scale)
        #pylab.legend(numpoints=1,bbox_to_anchor=(0.32, .22)) #Use this to align the label to the bottom left corner (after the scale)
        #pylab.legend(numpoints=1,bbox_to_anchor=(.42,1)) 
        #pylab.legend(numpoints=1,bbox_to_anchor=(1, .22)) #Use this to align the label to the bottom right corner (inside the plot)
        #pylab.legend(numpoints=1,bbox_to_anchor=(1.12, .22)) #Use this to align the label to the bottom right corner (outside the plot)
        #pylab.legend(numpoints=1)  #Only one point in the legend
    
    if (os.path.isfile(".plot.lock")==True):
        os.remove(".plot.lock")
        
    if (Graph.SaveFigure==True):
        pylab.savefig(Graph.SaveFigurePath)
    else:   
        pylab.show()
        
##########################################################################
# Name        : 
# Description : Main execution thread
##########################################################################
if (os.path.isfile(".plot.lock") != True) :
    PlotLockFile = open(".plot.lock", "w")   
    PlotLockFile.close()
  
Graph= Graphic()


HelpHeader = """
#############################################################################
# Copyright: gAGE/UPC & ESA
# Project: EDUNAV GNSS Lab Tool
# Authors: Adria Rovira-Garcia (group of Astronomy and GEomatics - gAGE/UPC)
#          Pere Ramos-Bosch (group of Astronomy and GEomatics - gAGE/UPC)
# Curator: Adria Rovira-Garcia ( gAGE/UPC )
#          glab.gage @ upc.edu             
# File: graph.py
# Code Management Tool File Version: 2.0  Revision: 00
# Date: 2012/12/31
#############################################################################


GRAPHIC GENERAL OPTIONS:

  The General Options are the ones that can be specified once per graphic,
  and will affect the entire graphic window:
  
  -h, --help                    Show this help message and exit
  
  -t, --title, --tit            Set the title of the Graphic
  
  --Xlabel, --xlab, --xl        Set the x-axis label
  
  --Ylabel, --ylab, --yl        Set the y-axis label
  
  --Xmin, --xmin, --xn          The minimum value for the x axis to be plot, if 
                                no value is provided, automatic limits are set.  
                                
  --Xmax, --xmax, --xx          The maximum value for the x axis to be plot, if 
                                no value is provided, automatic limits are set.
                                
  --Ymin, --ymin, --yn          The minimum value for the y axis to be plot, if 
                                no value is provided, automatic limits are set.
                                
  --Ymax, --ymax, --yx          The maximum value for the y axis to be plot, if 
                                no value is provided, automatic limits are set. 

  --Ftitle, --ftitle, --ft      Add a fractional title to the upper left corner.

  Example:
       graph.py -t "Graphic title" --xl "time (s)" --yl "Altitude [m]"
      --xmin -3.0 --ymax 5.0


PLOT DEPENDENT OPTIONS:

  The Plot Dependent Options are specific to each plot.
  One new plot is considered from the point that a '-f' or '--file' is found. 
  All the options coming after this parameter belong to that specific plot.
  
  -f, --file                    Set the input file name for the specific plot
  
  -x, -X, --Xcol                Set the source of the x axis: 
                                -x 4 or -x '$4' : will take as x axis the 4th
                                column of the input file.
                                
                                Operations can be done in this parameter, such as:
                                -x '($4-$5)': This will take the difference between
                                the 4th and the 5th column 
                                
                                Mathematical functions and constants are supported:
                                -x '(math.sin($12*math.pi/180)'
                                
  -y, -Y, --Ycol                Set the source of the y axis
                                Identical properties than x column
                                
  -c, --cond                    Specify the plotting condition:
                                -c '($6>=10)' Include the 6th column if it is 
                                equal or greater than 10
                                
                                -c '($1==\"OUTPUT\")' Include the 1st column by a specific
                                string: surrounding the string by \"  
                                
                                Mathematical functions, booleans and constants are supported:
                                -c ($1==\"POSTFIT\")&(math.e($12+$5)>5.0)
                                
  -l, --label, --plotlabel      Sets the label for the current plot
  
  -s, --style                   Sets the style for the current plot  
                                The following styles are supported:
                                '.'     circle marker [DEFAULT]
                                '-'     solid line style
                                '--'    dashed line style
                                '-.'    dash-dot line style
                                'o'     point marker
                                's'     square marker
                                'p'     pentagon marker
                                '+'     plus marker
                                'x'     x marker
                                
  --color, --cl                 Sets the colour for the current plot
                                The following colours are supported:
                                'b'     blue [DEFAULT]
                                'g'     green
                                'r'     red
                                'c'     cyan
                                'm'     magenta
                                'y'     yellow
                                'k'     black
                                'w'     white

  --markersize, --ms            Sets the plot maker size 5 by [DEFAULT]

  --linewidth, --lw             Sets the plot maker size 0 by [DEFAULT]

  Example:
       graph.py -f /home/gLAB/Example -x '(math.sin($1*math.pi/180))' -y 2 
       -l "Plot Label" --style p --color cyan                               
                                

MULTIPLE PLOTS:

  Each graphic window can contain several plots. One new plot is considered 
  from the point that a '-f' or '--file' is found:
      Graphic General Options: will apply to all the plots present
      Plot Dependent options: parameters specific to each plot.
  
  Example:
       graph.py -f /home/gLAB/Example -x '1' -y '2' -f /home/gLAB/Example -x '2' -y '1'


SAVE CURRENT GRAPHIC:

  User can save the entire graphic window into the path specified. If no path 
  is specified, it will be allocated in the current working directory.
  The graphic output format is deduced from the extension of the file name, if no
  extension is provided a 'png' extension type will be used: 
                         
  
  --save, --sv                  Stores the graphic window given a file name. The graphic
                                output format is deduced from the extension of the file name.
                                Supported formats are: 
                                emf, eps, jpeg, jpg, pdf, png [DEFAULT], ps, raw, rgba, svg, svgz.
                                
  Example:
       graph.py -f /home/gLAB/Example -x 1 -y 2 --save Figure_Example
       graph.py -f /home/gLAB/Example -x 1 -y 2 --sv /home/Desktop/Figure_Example.eps

"""

GraphicParser = OptionParser(usage=HelpHeader) 

GraphicParser.add_option("--title","--tit","-t", type="string", dest="Title", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ftitle","--ftitle","--ft", type="string", dest="FractionalTitle", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xlabel","--xlab","--xl" ,type="string", dest="Xlabel", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ylabel","--ylab","--yl" ,type="string", dest="Ylabel", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--file","-f" ,type="string", dest="filename", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xcol","-x","-X", type="string", dest="ColX", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ycol","-y","-Y", type="string",dest="ColY", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--cond","-c", type="string",dest="Cond", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--label","--plotlabel","-l", type="string",dest="PlotLabel", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--style","-s", type="string",dest="Style", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP) 

GraphicParser.add_option("--ms","--MarkerSize","--markersize",type="string",dest="MarkerSize", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP) 
GraphicParser.add_option("--lw","--LineWidth","--linewidth", type="string",dest="LineWidth", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP) 

GraphicParser.add_option("--save","--sv", type="string",dest="SaveFigure", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--color","--cl", type="string",dest="PlotColor", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xmin","--xmin","--xn", type="string",dest="Xmin", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xmax","--xmax","--xx", type="string",dest="Xmax", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ymin","--ymin","--yn", type="string",dest="Ymin", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ymax","--ymax","--yx", type="string",dest="Ymax", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)

GraphicParser.parse_args()


MakeGraph()
